@@ -65,12 +65,15 @@ class ConfigurationProvider(object):
         else:
             f = filename
             newfile = False
+        exception = None
         try:
             self._write(f, config)
-        finally:
-            # XXX try: finally: works in python >= 2.5
-            if newfile:
-                f.close()
+        except Exception, exception:
+            pass
+        if newfile:
+            f.close()
+        if exception:
+            raise exception
     def _write(self, fp, config):
         raise NotImplementedError("Abstract base class")
 
@@ -111,19 +114,21 @@ class ConfigurationOption(optparse.Option):
     """option that keeps track if it is seen"""
     # TODO: this should be configurable or something
     def take_action(self, action, dest, opt, value, values, parser):
-        optparse.Option.take_action(self, action, dest, opt, value, values, parser)
-
-        # add the parsed option to the set of things parsed
-        if not hasattr(parser, 'parsed'):
-            parser.parsed = set()
-        parser.parsed.add(dest)
 
         # switch on types
         formatter = getattr(parser, 'cli_formatter')
         if formatter:
             formatter = formatter(dest)
             if formatter:
-                setattr(values, dest, formatter(getattr(values, dest)))
+                value = formatter(value)
+
+        # call the optparse front-end
+        optparse.Option.take_action(self, action, dest, opt, value, values, parser)
+
+        # add the parsed option to the set of things parsed
+        if not hasattr(parser, 'parsed'):
+            parser.parsed = dict()
+        parser.parsed[dest] = value
 
 ### plugins for option types
 
@@ -202,11 +207,20 @@ class DictCLI(ListCLI):
 
     delimeter = '='
 
+    def __call__(self, name, value):
+
+        # optparse can't handle dict types OOTB
+        default = value.get('default')
+        if isinstance(default, dict):
+            value = copy.deepcopy(value)
+            value['default'] = default.items()
+
+        return ListCLI.__call__(self, name, value)
+
     def take_action(self, value):
-        bad = [i for i in value if self.delimeter not in i]
-        if bad:
-            raise AssertionError("Each value must be delimited by '%s': %s" % (self.delimeter, bad))
-        return dict([i.split(self.delimeter, 1) for i in value])
+        if self.delimeter not in value:
+            raise AssertionError("Each value must be delimited by '%s': %s" % (self.delimeter, value))
+        return value.split(self.delimeter, 1)
 
 # TODO: 'dict'-type cli interface
 
@@ -257,7 +271,7 @@ class Configuration(optparse.OptionParser):
                 parser_args['formatter'] = PlainDescriptionFormatter()
         parser_args.setdefault('option_class', ConfigurationOption)
         optparse.OptionParser.__init__(self, **parser_args)
-        self.parsed = set()
+        self.parsed = dict()
         self.optparse_options(self)
         # add option(s) for configuration_providers
         if load:
@@ -271,8 +285,8 @@ class Configuration(optparse.OptionParser):
             if isinstance(dump, basestring):
                 dump = [dump]
             dump = list(dump)
-            self.add_option(*dump, dest='dump',
-                            help="Output configuration file; Formats: %s" % formats)
+            self.add_option(*dump, **dict(dest='dump',
+                                          help="Output configuration file; Formats: %s" % formats))
 
 
     ### methods for iteration
@@ -376,7 +390,7 @@ class Configuration(optparse.OptionParser):
 
     def cli_formatter(self, option):
         if option in self.option_dict:
-            handler = self.types[self.option_dict[option].get('type')]
+            handler = self.types[self.option_type(option)]
             return getattr(handler, 'take_action', lambda x: x)
 
     def option_type(self, name):
@@ -410,7 +424,7 @@ class Configuration(optparse.OptionParser):
 
     def parse_args(self, *args, **kw):
 
-        self.parsed = set()
+        self.parsed = dict()
         options, args = optparse.OptionParser.parse_args(self, *args, **kw)
 
         # get CLI configuration options
@@ -446,7 +460,7 @@ class Configuration(optparse.OptionParser):
             pass
 
         # dump configuration
-        dump = self.dump(options, missingvalues)
+        self.dump(options, missingvalues)
 
         # update options from config
         options.__dict__.update(self.config)
